@@ -13,7 +13,7 @@
 // and `-s NO_OPTION` expands to `-s OPTION=0` (assuming OPTION is a valid
 // option).
 //
-// See https://github.com/kripken/emscripten/wiki/Code-Generation-Modes/
+// See https://github.com/emscripten-core/emscripten/wiki/Code-Generation-Modes/
 //
 // Note that the values here are the defaults in -O0, that is, unoptimized
 // mode. See apply_opt_level in tools/shared.py for how -O1,2,3 affect these
@@ -557,7 +557,7 @@ var EXCEPTION_CATCHING_WHITELIST = [];
 // or programs that don't need exit() to work.
 
 // For more explanations of this option, please visit
-// https://github.com/kripken/emscripten/wiki/Asyncify
+// https://github.com/emscripten-core/emscripten/wiki/Asyncify
 var NODEJS_CATCH_EXIT = 1;
 
 // Whether to enable asyncify transformation
@@ -688,7 +688,14 @@ var PROFILING_FUNCS = 0;
 // may be slightly misleading, as this is for any JS library element, and not
 // just functions. For example, you can include the Browser object by adding
 // "$Browser" to this list.
-var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = ['memcpy', 'memset', 'malloc', 'free'];
+var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = [
+	'memcpy',
+	'memset',
+	'malloc',
+	'free',
+	'emscripten_get_heap_size', // Used by dynamicAlloc() and -s FETCH=1
+	'emscripten_resize_heap' // Used by dynamicAlloc() and -s FETCH=1
+	];
 
 // This list is also used to determine auto-exporting of library dependencies
 // (i.e., functions that might be dependencies of JS library functions, that if
@@ -1350,7 +1357,6 @@ var EMIT_EMSCRIPTEN_METADATA = 0;
 // for effective code size optimizations to take place.
 var SUPPORT_ERRNO = 1;
 
-
 // Internal: points to a file that lists all asm.js/wasm module exports, annotated
 // with suppressions for Closure compiler, that can be passed as an --externs file
 // to Closure.
